from django.conf import settings


def MAX_USERNAME_LENGTH():
    return getattr(settings, "MAX_USERNAME_LENGTH", 255)


def MAX_EMAIL_LENGTH():
    return getattr(settings, "MAX_EMAIL_LENGTH", 255)


def REQUIRE_UNIQUE_EMAIL():
    return getattr(settings, "REQUIRE_UNIQUE_EMAIL", True)



# encoding: utf-8
from south.db import db
from south.v2 import SchemaMigration
from django.db import models
from longerusernameandemail import MAX_USERNAME_LENGTH, MAX_EMAIL_LENGTH, REQUIRE_UNIQUE_EMAIL


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Changing field 'User.username'
        db.alter_column('auth_user', 'username', models.CharField(max_length=MAX_USERNAME_LENGTH()))
        # Increase length of email field to match
        db.alter_column('auth_user', 'email', models.CharField(max_length=MAX_EMAIL_LENGTH()))
        # Add an index to make email field unique
        db.create_index('auth_user', ['email'], unique=REQUIRE_UNIQUE_EMAIL())

    def backwards(self, orm):

        # Changing field 'User.username'
        db.alter_column('auth_user', 'username', models.CharField(max_length=35))
        db.alter_column('auth_user', 'email', models.CharField(max_length=75))
        db.delete_index('auth_user', ['email'],)

    models = {

    }

    complete_apps = ['django_monkeypatches']



from django.core.validators import MaxLengthValidator
from django.db.models.signals import class_prepared
from django.utils.translation import ugettext as _
from longerusernameandemail import MAX_USERNAME_LENGTH, REQUIRE_UNIQUE_EMAIL


def longer_username_and_email_signal(sender, *args, **kwargs):
    if (sender.__name__ == "User" and
        sender.__module__ == "django.contrib.auth.models"):
        patch_user_model_username(sender)
        patch_user_model_email(sender)
class_prepared.connect(longer_username_and_email_signal)


def patch_user_model_username(model):
    field = model._meta.get_field("username")

    field.max_length = MAX_USERNAME_LENGTH()
    field.help_text = _("Required, %s characters or fewer. Only letters, "
                        "numbers, and @, ., +, -, or _ "
                        "characters." % MAX_USERNAME_LENGTH())

    # patch model field validator because validator doesn't change if we change
    # max_length
    for v in field.validators:
        if isinstance(v, MaxLengthValidator):
            v.limit_value = MAX_USERNAME_LENGTH()


def patch_user_model_email(model):
    field = model._meta.get_field("email")

    field.blank = False
    field._unique = REQUIRE_UNIQUE_EMAIL()
    field.max_length = MAX_USERNAME_LENGTH()
    field.help_text = _("Required, %s characters or fewer. Only letters, "
                        "numbers, and @, ., +, -, or _ "
                        "characters." % MAX_USERNAME_LENGTH())

    # patch model field validator because validator doesn't change if we change
    # max_length
    for v in field.validators:
        if isinstance(v, MaxLengthValidator):
            v.limit_value = MAX_USERNAME_LENGTH()


from django.contrib.auth.models import User

# https://github.com/GoodCloud/django-longer-username/issues/1
# django 1.3.X loads User model before class_prepared signal is connected
# so we patch model after it's prepared

# check if User model username is patched
if User._meta.get_field("username").max_length != MAX_USERNAME_LENGTH():
    patch_user_model_username(User)

# check if user model email is patched
if User._meta.get_field("email").max_length != MAX_USERNAME_LENGTH():
    patch_user_model_email(User)



import random
import string

from django.contrib.auth.models import User
from django.test import TestCase

from longerusernameandemail.forms import UserCreationForm


def get_random_string(alpha=string.ascii_letters, length=100):
    "Get a 'long', randmon string"
    return ''.join([random.choice(alpha) for i in range(length)])


class LongerUsernameAndEmailTests(TestCase):
    """
    Unit tests for longerusernameandemail app
    """

    def setUp(self):
        """
        creates a user with a terribly long username
        """
        long_username = ''.join([str(i) for i in range(100)])

        self.user = User.objects.create_user(
            'test%s' % long_username,
            '%s@example.com' % long_username,
            'testpassword'
        )

    def testUserCreation(self):
        """
        tests that self.user was successfully saved, and can be retrieved
        """
        self.assertNotEqual(self.user, None)

        # returns DoesNotExist error if the user wasn't created
        User.objects.get(id=self.user.id)


class LongerUsernameAndEmailFormTests(TestCase):
    """
    Unit tests for longerusernameandemail forms.
    """

    def setUp(self):
        # create a user with long username & email
        self.user = User.objects.create_user(
            'test%s' % get_random_string(),
            '%s@example.com' % get_random_string(),
            'testpassword',
        )

    def test_valid_new_user(self):
        "test a new user with a long username and long email is valid"
        data = {
            'username': get_random_string(),
            'email': '%s@example2.com' % get_random_string(),
            'password1': 'test',
            'password2': 'test',
        }
        form = UserCreationForm(data)
        self.assertTrue(form.is_valid())

    def test_invalid_new_user_email_collision(self):
        """
        test we can't create a new user with the same email as an
        existing user
        """
        data = {
            'username': 'anything',
            'email': self.user.email,
            'password1': 'test',
            'password2': 'test',
        }
        form = UserCreationForm(data)
        self.assertFalse(form.is_valid())
        self.assertIn('email', form.errors)
        self.assertIn('already exists', form.errors['email'][0])


class LongerUsernameAndEmailAdminTests(TestCase):
    """
    Functional tests for the django admin when longerusernameandemail
    is enabled.
    """

    urls = 'longerusernameandemail.tests.urls'

    email = 'superuserusername@example.com'
    password = 'superuserpassowrd'

    def setUp(self):
        # create two users with long usernames & emails
        self.user1 = User.objects.create_user(
            'test%s' % get_random_string(),
            '%s@example.com' % get_random_string(),
            'testpassword',
        )
        self.user2 = User.objects.create_user(
            'test%s' % get_random_string(),
            '%s@example.com' % get_random_string(),
            'testpassword',
        )

        # create superuser to do the actions, and log in as them
        User.objects.create_superuser(self.email, self.email, self.password)
        self.client.login(username=self.email, password=self.password)

    def test_read_user_list(self):
        "test we can read the list of users in the admin"
        resp = self.client.get('/admin/auth/user/')
        self.assertEqual(resp.status_code, 200)

    def test_read_user(self):
        "test we can read a particular user in the admin"
        url = '/admin/auth/user/{}/'.format(self.user1.pk)
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 200)

    def test_create_user(self):
        "test we can create a new user using the admin"
        org_user_count = User.objects.count()
        resp = self.client.post('/admin/auth/user/add/', data={
            'username': 'test{}@example.com'.format(get_random_string()),
            'email': 'test{}@example.com'.format(get_random_string()),
            'password1': 'test',
            'password2': 'test',
        })
        self.assertEqual(resp.status_code, 302)
        self.assertEqual(User.objects.count(), org_user_count + 1)

    def test_edit_user(self):
        "test we can edit a particular user using the admin"
        new_email = 'test{}@example.com'.format(get_random_string())
        url = '/admin/auth/user/{}/'.format(self.user1.pk)
        resp = self.client.post(url, {
            'username': new_email,
            'email': new_email,
            'last_login_0': self.user1.last_login.strftime('%F'),
            'last_login_1': self.user1.last_login.strftime('%T'),
            'date_joined_0': self.user1.date_joined.strftime('%F'),
            'date_joined_1': self.user1.date_joined.strftime('%T'),
        })
        self.assertEqual(resp.status_code, 302)
        self.assertEqual(User.objects.filter(email=new_email).count(), 1)

    def test_delete_user(self):
        "test we can delete a new user using the admin"
        user = User.objects.create_user('t@e.com', 't@e.com', 'pwd')
        org_user_count = User.objects.count()
        url = '/admin/auth/user/{}/delete/'.format(user.pk)
        resp = self.client.post(url, {'post': 'yes'})
        self.assertEqual(resp.status_code, 302)
        self.assertEqual(User.objects.count(), org_user_count - 1)



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.models import User

from longerusernameandemail.forms import UserCreationForm, UserChangeForm


class LongerUsernameAndEmailUserAdmin(UserAdmin):
    add_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('username', 'email', 'password1', 'password2')}
        ),
    )
    add_form = UserCreationForm
    form = UserChangeForm


admin.site.unregister(User)
admin.site.register(User, LongerUsernameAndEmailUserAdmin)



#/usr/bin/env python
import os
from setuptools import setup, find_packages


ROOT_DIR = os.path.dirname(__file__)
SOURCE_DIR = os.path.join(ROOT_DIR)


setup(
    name="django-longerusernameandemail",
    version="0.5.6",
    packages=find_packages(),
    zip_safe=False,
    description=("django-longerusernameandemail provides a migration and a "
                 "monkeypatch to make the django auth.user username and email "
                 "fields longer, instead of the arbitrarily short 30 and 75 "
                 "characters."),
    license="BSD",
    keywords="django long email username migration south",
)
